/*This program is for doing the cannonical ensemble on the torus*/

/*includes*/
#include"mc_state_h.h"
#include"cell_list_h.h"
#include"monte_carlo_h.h"
#include"mc_math_h.h"
#include"std_h.h"
#include"twister_h.h"
#include"read_h.h"
#include"running_average_h.h"
#include"energy_h.h"
#include"histograms_h.h"
#include"histograms2D_h.h"
#include"neighbours_h.h"
#include"torus_cluster_angle_h.h"
#include"histograms_2D_continuous_h.h"
#include"angular_distribution_h.h"
#include"energy_angle_h.h"


int main (int argc, char *argv[]) {

/*validation*/
if (argc != 2) die ("This program requires one argument, the run ID");

/*system structs*/
Mc_state state;
Mc_cell cell;
Data input;
Mc_average energy_average;
Mc_average liquid_average;
Mc_average bond_average;
Mc_histogram energy_histogram;
Mc_histogram neighbours_histogram;
Mc_histogram angle_histogram;
Mc_histogram bond_histogram;
Mc_hist2DC bond_angle2D;
Mc_hist2DC bond_angle_free2D;
EA_function en_function;

/*read options file*/
char command[50];
FILE *fp;
sprintf(command, "options%s.txt",argv[1]);
fp = fopen(command,"r");
READ_options(&input,fp);
fclose(fp);

/*seed the random number generator*/
seedMT(input.seed);

/*fix file*/
sprintf(command,"mkdir run%s",argv[1]);
system(command);
sprintf(command,"cp options%s.txt run%s", argv[1],argv[1]);
system(command);
sprintf(command, "run%s",argv[1]);
chdir(command);
printf("hi\n");
int i,j;
/*set up variables for stuff that'll be used a lot (for speed)*/
long max_steps = input.max_steps;
long record_rate = input.record_rate;
long average_rate = input.average_rate;
long frame_rate = input.frame_rate;
long equilibriate = input.equilibriate;
int periodic = input.periodic;
double delta_r = input.delta_r;
double *paramsP = input.paramsP;
double *paramsG = input.paramsG;
double neighbour_cutoff = input.neighbour_cutoff;
int n_max = input.n_max;
double order_cutoff = input.order_cutoff;
void (*move)(double *x, double *y, double *z, double *paramsD, double delta_r);
void (*random) (double *x, double *y, double *z, double *params);
double (*get_area) (double params[]);
double (*potential) (double distance, double params[]);
double * (*BO6_calculate) (Mc_state *state, Mc_cell *cell, double *paramsG, double cutoff);
double deltaC_move=0;
double deltaC_spin=0;
move = input.move;
random = input.random;
potential = input.potential;
get_area = input.get_area;
BO6_calculate = input.BO6_calculate;

/*initialise*/
mc_initialise(&state,input.read_file,input.restart,input.n_particles_initial,
		paramsG,argv[1],random,&delta_r,&deltaC_move,&deltaC_spin);
CLL_init(&cell, input.cell_box_edge, input.cell_min_size);
CLL_init_particles(&cell,&state);

RA_new_average(&energy_average, "energy",max_steps/record_rate,input.restart);
RA_new_average(&liquid_average,"liquid",max_steps/record_rate,input.restart);
RA_new_average(&bond_average,"bond", max_steps/record_rate,input.restart);
HG_new_histogram(&energy_histogram,0.0,-8.0,0.1,"energy",input.restart);
HG_new_histogram(&neighbours_histogram,10.0,0.0,1.0,"neighbours",input.restart);
HG_new_histogram(&angle_histogram,2*M_PI,0.0,0.03,"angle_hist",input.restart);
HG_new_histogram(&bond_histogram,1.4,0.0,0.01,"bond",input.restart);
EA_new_function(&en_function);
H2DC_new_histogram(&bond_angle2D,
                  (float) state.n_particles,0.0,1.0,2*M_PI,0.0,0.01*M_PI,"bond_angle",input.restart);
H2DC_new_histogram(&bond_angle_free2D,
                  (float) state.n_particles,0.0,1.0,
                  (float) state.n_particles,0.0,1.0,"bond_angle_free",input.restart);

float rm;
/*------------------------------equilibriate-------------------------------------*/

long steps = 0;
int total_moves;
int moves_accepted;
double *order;
while (steps < equilibriate) {
	total_moves = 0;
	moves_accepted = 0;

	for (i = 0; i < average_rate; i++) {
		rm = randomF();
		total_moves++;
		if(mc_particle_move(&state,&cell,periodic,delta_r,potential, move, paramsP, paramsG)){
			moves_accepted++;
		}

	}

	steps += average_rate;
	if ((float) moves_accepted/total_moves > 0.25) {
		if (delta_r < paramsG[0]) delta_r *= 1.05;
	} else if ((float) moves_accepted/total_moves < 0.25) {
		delta_r /= 1.05;
	}

	if (steps % frame_rate == 0) {
		mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
	}
}

steps = 0;
/*---------------------------simulation loop-------------------------------------*/
int count_records = 0;
double *en_temp;
double *nei_temp;
double *ang_temp;
double ang_av;
double *bond_temp;
double en_tot;
double liq_tot;
double bond_tot;
int orderA,orderB;
while (steps < max_steps) {
	
	for (i = 0; i < average_rate; i++) {
		rm = randomF();
		mc_particle_move(&state,&cell,periodic,delta_r,potential, move, paramsP, paramsG);

	}

	steps += average_rate;

	if (steps % average_rate == 0) {
		count_records++;

		/*-----------------do experiments----------------*/
	en_temp = EGY_energy_list (&state, &cell,periodic,paramsG, potential, paramsP);
	en_tot = array_sum_d(en_temp,state.n_particles);
	nei_temp = NEI_double(&state,paramsG,neighbour_cutoff,periodic);
	liq_tot = NEI_liquid(&state,paramsG,neighbour_cutoff,order_cutoff,periodic);
	bond_temp = BO6_calculate(&state,&cell, paramsG, neighbour_cutoff);
	bond_tot = array_sum_d(bond_temp,state.n_particles)/state.n_particles;
	HG_add_array(&energy_histogram,en_temp,state.n_particles);
	HG_add_array(&neighbours_histogram,nei_temp,state.n_particles);
	HG_add_array(&bond_histogram,bond_temp,state.n_particles);
	RA_add_average(&energy_average,en_tot);
	RA_add_average(&liquid_average,liq_tot);
	RA_add_average(&bond_average,bond_tot);
	/*measure angle*/
	ang_temp = ANG_get_angles(&state,paramsG);
	ang_av = TAK_get_angle(&state,paramsG);
	HG_add_value(&angle_histogram,ang_av);
	EA_add_value(&en_function, en_temp,ang_temp,state.n_particles);
	orderA = state.n_particles - threshold_count(ang_temp,state.n_particles,M_PI/2);
	orderB = threshold_count(bond_temp,state.n_particles,0.6);
	H2DC_add_value(&bond_angle2D,orderB,ang_av);
	H2DC_add_value(&bond_angle_free2D,orderB,orderA);

	if (steps % record_rate == 0) {
		RA_push_average(&energy_average);
		RA_push_average(&liquid_average);
		RA_push_average(&bond_average);
	}
	
	if (steps % frame_rate == 0) {
		mc_record(&state);
		mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
		RA_print_average(&energy_average);
		RA_print_average(&liquid_average);
		RA_print_average(&bond_average);
		HG_print_histogram(&energy_histogram);
		HG_print_histogram(&neighbours_histogram);
		HG_print_histogram(&bond_histogram);
		HG_print_histogram(&angle_histogram);
		EA_print(&en_function);
		H2DC_print_histogram(&bond_angle2D);
		H2DC_print_histogram(&bond_angle_free2D);
		mc_record_order( nei_temp, state.n_particles,"neighbours");
		mc_record_order( bond_temp, state.n_particles, "bond_order");
	}

	free(nei_temp);
	free(en_temp);
	free(ang_temp);
	free(bond_temp);

		/*just in case rounding errors*/
	if (count_records > (max_steps/average_rate) - 1) break;
	}
}

/*-------------------------save the data----------------------------*/
fp = fopen("data.txt","w");
fprintf(fp,"Records %d",count_records);
fclose(fp);
mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
RA_push_average(&energy_average);
RA_push_average(&liquid_average);
RA_push_average(&bond_average);
RA_print_average(&energy_average);
RA_print_average(&liquid_average);
RA_print_average(&bond_average);
HG_print_histogram(&energy_histogram);
HG_print_histogram(&neighbours_histogram);
HG_print_histogram(&angle_histogram);
HG_print_histogram(&bond_histogram);
H2DC_print_histogram(&bond_angle2D);
H2DC_print_histogram(&bond_angle_free2D);
EA_print(&en_function);
CLL_end(&cell);
MCS_end(&state);

READ_destroy(&input);

return 0;
}
