/*This program is for doing crystalline umbrella sampling of the cannonical ensemble on the sphere*/

/*includes*/
#include"mc_state_h.h"
#include"cell_list_h.h"
#include"monte_carlo_h.h"
#include"mc_math_h.h"
#include"std_h.h"
#include"twister_h.h"
#include"read_h.h"
#include"running_average_h.h"
#include"energy_h.h"
#include"histograms_h.h"
#include"histograms2D_h.h"
#include"neighbours_h.h"
#include"umbrella_basic_h.h"
#include"angular_distribution_h.h"
#include"energy_angle_h.h"

int main (int argc, char *argv[]) {

/*validation*/
if (argc != 2) die ("This program requires one argument, the run ID");

/*system structs*/
Mc_state state;
Mc_state old_state;
Mc_cell cell;
Mc_cell old_cell;
Data input;
Mc_umbrellaB umbrella;

/*read options file*/
char command[50];
FILE *fp;
sprintf(command, "options%s.txt",argv[1]);
fp = fopen(command,"r");
READ_options(&input,fp);
fclose(fp);

/*seed the random number generator*/
seedMT(input.seed);

/*fix file*/
sprintf(command,"mkdir run%s",argv[1]);
system(command);
sprintf(command,"cp options%s.txt run%s", argv[1],argv[1]);
system(command);
sprintf(command, "run%s",argv[1]);
chdir(command);

int i,j;
/*set up variables for stuff that'll be used a lot (for speed)*/
long max_steps = input.max_steps;
long record_rate = input.record_rate;
long average_rate = input.average_rate;
long frame_rate = input.frame_rate;
long equilibriate = input.equilibriate;
int periodic = input.periodic;
double delta_r = input.delta_r;
double deltaC_move = input.deltaC_move;
double deltaC_spin = input.deltaC_spin;
double *paramsP = input.paramsP;
double *paramsG = input.paramsG;
double neighbour_cutoff = input.neighbour_cutoff;
double lambda = input.lambda;
int n_target = input.n_target;
long umbrella_rate = input.umbrella_rate;
double order_cutoff = input.order_cutoff;
void (*move)(double *x, double *y, double *z, double *paramsD, double delta_r);
void (*random) (double *x, double *y, double *z, double *params);
double (*get_area) (double params[]);
double (*potential) (double distance, double params[]);
move = input.move;
random = input.random;
potential = input.potential;
get_area = input.get_area;

/*initialise*/
mc_initialise(&state,input.read_file,input.restart,input.n_particles_initial,
		paramsG,argv[1],random,&delta_r,&deltaC_move,&deltaC_spin);
CLL_init(&cell, input.cell_box_edge, input.cell_min_size);
CLL_init_particles(&cell,&state);
MCS_state_clone(&old_state,&state);
CLL_cell_clone(&old_cell,&cell,state.n_particles);

UBB_new_histogram(&umbrella,get_area(paramsG),"Nucleus",input.restart);

long steps = 0;
int total_moves,count_records = 0;
int moves_accepted;
double *angles;
int order;
int old_order;
double rm;

while (steps < equilibriate) {

	total_moves = 0;
	moves_accepted = 0;

	for (i = 0; i < umbrella_rate; i++) {
		rm = randomF();
		total_moves++;
		if(mc_particle_move(&state,&cell,periodic,delta_r,potential, move, paramsP, paramsG)){
			moves_accepted++;
		}
	}
	
	if (steps > umbrella_rate * 100) {
		angles = ANG_get_angles(&state,paramsG);
		order = state.n_particles-threshold_count(angles,state.n_particles,M_PI/2);
		UBB_move(&state,&old_state,&cell,&old_cell,&order,&old_order,lambda,n_target);
		free(angles);
	}
	steps += umbrella_rate;
	if ((float) moves_accepted/total_moves > 0.25) {
		if (delta_r < paramsG[0]) delta_r *= 1.05;
	} else if ((float) moves_accepted/total_moves < 0.25) {
		delta_r /= 1.05;
	}

	if (steps % frame_rate == 0) {
		mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
	}
}

double *nei_temp;
while (steps < max_steps) {
	
	for (i = 0; i < umbrella_rate; i++) {
		rm = randomF();
		total_moves++;
		if(mc_particle_move(&state,&cell,periodic,delta_r,potential, move, paramsP, paramsG)){
			moves_accepted++;
		}
	}
	
	
	angles = ANG_get_angles(&state,paramsG);
	order = state.n_particles - threshold_count(angles,state.n_particles,M_PI/2);
	UBB_move(&state,&old_state,&cell,&old_cell,&order,&old_order,lambda,n_target);
	free(angles);
	
	steps += umbrella_rate;

	if (steps % average_rate == 0) {
		UBB_add_value(&umbrella,order);
		if (steps % frame_rate == 0) {
			UBB_print(&umbrella);
			mc_record(&state);
			mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
			nei_temp = NEI_double(&state,paramsG,neighbour_cutoff,periodic);
			mc_record_order( nei_temp, state.n_particles,"neighbours");
			free(nei_temp);	
		}
		/*just in case rounding errors*/
		if (count_records > (max_steps/average_rate) - 1) break;
	}
}

/*-------------------------save the data----------------------------*/
fp = fopen("data.txt","w");
fprintf(fp,"Records %d",count_records);
fclose(fp);
mc_save(&state,argv[1],delta_r,deltaC_move,deltaC_spin);
UBB_scale(&umbrella,lambda,n_target);
UBB_print(&umbrella);
return 0;
}
