/*A test that does the most bog standard possible mc simulation on a sphere.
 *It takes no inputs and only gives very rudimentry outputs.
 */


#include"mc_state_h.h"
#include"cell_list_h.h"
#include"monte_carlo_h.h"
#include"mc_math_h.h"
#include"std_h.h"
#include"twister_h.h"
#include"read_h.h"
#include"running_average_h.h"
#include"energy_h.h"
#include"histograms_h.h"
#include"histograms2D_h.h"
#include"bond_order_6h.h"
#include"neighbours_h.h"
#include"umbrella_h.h"
#include"angular_distribution_h.h"
#include"energy_angle_h.h"

int main (int argc, char *argv[]) {

/*check that there is one input argument*/
if (argc != 2) {
	die("This program requires one argument, the run ID");
}

/*system*/
Mc_state state;
Mc_cell cell;
Data input;
Mc_average energy_average;
Mc_average num_average;
Mc_histogram num_histogram;
Mc_histogram energy_histogram;
Mc_histogram angle_histogram;
Mc_average bond_average;
Mc_histogram bond_histogram;
EA_function en_function;
Mc_hist2D num_eng;
Mc_umbrella umbrella;

/*read options file*/
char command[50];
FILE *fp;
sprintf(command, "options%s.txt",argv[1]);
fp = fopen(command,"r");
READ_options(&input,fp);
fclose(fp);

/*fix file*/
sprintf(command,"mkdir run%s",argv[1]);
system(command);
sprintf(command,"cp options%s.txt run%s", argv[1],argv[1]);
system(command);
sprintf(command, "run%s",argv[1]);
chdir(command);


int i,j;
/*set up variables for stuff that'll be used a lot (for speed)*/
long max_steps = input.max_steps;
long record_rate = input.record_rate;
long average_rate = input.average_rate;
long frame_rate = input.frame_rate;
long equilibriate = input.equilibriate;
double delta_r = input.delta_r;
double deltaC_move = input.deltaC_move;
double deltaC_spin = input.deltaC_spin;
double *paramsG = input.paramsG;
double *paramsP = input.paramsP;
int periodic = input.periodic;
double activity = input.activity;
double neighbour_cutoff = input.neighbour_cutoff;
double order_cutoff = input.order_cutoff;
void (*move)(double *x, double *y, double *z, double *paramsD, double delta_r);
void (*random) (double *x, double *y, double *z, double *params);
double (*potential) (double distance, double params[]);
double (*get_area) (double params[]);
double * (*BO6_calculate) (Mc_state *state, Mc_cell *cell, double *paramsG, double cutoff);
move = input.move;
random = input.random;
get_area = input.get_area;
potential = input.potential;
BO6_calculate = input.BO6_calculate;


/*initialise*/
seedMT(input.seed);
mc_initialise(&state,input.read_file,input.restart,input.n_particles_initial,
		paramsG,argv[1],random,&delta_r,&deltaC_move,&deltaC_spin);
CLL_init(&cell, input.cell_box_edge, input.cell_min_size);
CLL_init_particles(&cell,&state);

RA_new_average(&energy_average, "energy",max_steps/record_rate,input.restart);
RA_new_average(&num_average, "number", max_steps/record_rate,input.restart);
RA_new_average(&bond_average,"bond", max_steps/record_rate,input.restart);
HG_new_histogram(&energy_histogram,0.0,-8.0,0.1,"energy",input.restart);
HG_new_histogram(&bond_histogram,1.4,0.0,0.01,"bond",input.restart);
HG_new_histogram(&num_histogram,1500.5,-0.5,1.0,"number",input.restart);
HG_new_histogram(&angle_histogram,M_PI,0.0,0.03,"angle",input.restart);
EA_new_function(&en_function);
sprintf(command,"e%f_a%fsamp",paramsP[0],activity);
H2D_new_histogram(&num_eng, 0.0, -8.0, 0.1, 2000, command, input.restart);
UMB_new_histogram(&umbrella,get_area(paramsG),"Nucleus",input.restart);

float rm;
/*-----------------Equilibirate-------------------*/
int total_moves;
int moves_accepted;
int total_adds;
int adds_accepted;
int total_subs;
int subs_accepted;
long steps = 0;
while (steps < equilibriate) {
	total_moves = 0;
	moves_accepted = 0;
	total_adds = 0;
	adds_accepted = 0;
	total_subs = 0;
	subs_accepted = 0;
	

	for (i = 0; i < 5000; i++) {
		rm = randomF();
		if (rm < 0.5) {
			total_moves ++;
			if (mc_particle_move(&state,&cell,periodic,
					    delta_r,potential, move, paramsP, paramsG)) {
				moves_accepted++;
			}
		} else if (rm < 0.75) {
			total_adds ++;
			if (mc_particle_add (&state,&cell,periodic,potential,random,paramsP,
						paramsG,activity)){
				adds_accepted ++;
			}
		} else {
			total_subs++;
			if(mc_particle_remove(&state,&cell,periodic,potential,
						 paramsP, paramsG, activity)) {
				subs_accepted ++;
			}
		}
	}

	steps += 5000;

	if ((float) moves_accepted/total_moves > 0.25) {
		if (delta_r < paramsG[0]) delta_r *= 1.05;
	} else if ((float) moves_accepted/total_moves < 0.25) {
		delta_r /= 1.05;
	}

	if (steps % frame_rate == 0) {
		mc_save(&state,argv[1],delta_r,0.0,0.0);
	}
	

}

/*----------------------------simulation loop----------------------------------*/
steps = 0;
int count_records = 0;
double *en_temp;
double *bond_temp;
double *nei_temp;
double * cluster_temp;
double * ang_temp;
double en_tot;
double bond_tot;
double *nuclei;
int *nuclei_int;
while (steps < max_steps) {

	for (i = 0; i < average_rate; i++) {
		rm = randomF();
		if (rm < 0.5) {
			mc_particle_move(&state,&cell,periodic,
					    delta_r,potential, move, paramsP, paramsG);
		} else if (rm < 0.75) {
			mc_particle_add(&state,&cell,periodic,potential,random,
						paramsP,paramsG,activity);
		} else {
			mc_particle_remove(&state,&cell,periodic,potential,
						paramsP, paramsG, activity);
		}		
	}
	steps += average_rate;
	count_records ++;
	/*----------------do experiments--------------------------*/
	en_temp = EGY_energy_list (&state, &cell,periodic,paramsG, potential, paramsP);
	en_tot = array_sum_d(en_temp,state.n_particles);
	bond_temp = BO6_calculate(&state,&cell, paramsG, neighbour_cutoff);
	bond_tot = array_sum_d(bond_temp,state.n_particles)/state.n_particles;
	nei_temp = NEI_double(&state, paramsG, neighbour_cutoff,periodic);
	ang_temp = ANG_get_angles(&state,paramsG);
	HG_add_array(&energy_histogram,en_temp,state.n_particles);
	HG_add_array(&bond_histogram,bond_temp,state.n_particles);
	HG_add_value(&num_histogram,(double) state.n_particles);
	HG_add_array(&angle_histogram,ang_temp,state.n_particles);
	H2D_add_value(&num_eng,en_tot, state.n_particles);
	RA_add_average(&energy_average,en_tot);
	RA_add_average(&bond_average,bond_tot);
	RA_add_average(&num_average,(double) state.n_particles);
	EA_add_value(&en_function,en_temp,ang_temp,state.n_particles);
	nuclei_int = UMB_get_nucleus(&state,order_cutoff,neighbour_cutoff,nei_temp,periodic,paramsG);
	UMB_add_array(&umbrella, nuclei_int, state.n_particles);

	if (steps % record_rate == 0) {
		RA_push_average(&energy_average);
		RA_push_average(&num_average);
		RA_push_average(&bond_average);
	}

	if (steps % frame_rate == 0) {
		mc_record(&state);
		mc_save(&state,argv[1],delta_r,0.0,0.0);
		RA_print_average(&energy_average);
		RA_print_average(&num_average);
		RA_print_average(&bond_average);
		HG_print_histogram(&energy_histogram);
		HG_print_histogram(&angle_histogram);
		HG_print_histogram(&bond_histogram);
		HG_print_histogram(&num_histogram);
                EA_print(&en_function);
		H2D_print_histogram(&num_eng,get_area(paramsG));
		cluster_temp = UMB_record_largest(&state,order_cutoff,neighbour_cutoff,
							nei_temp, periodic, paramsG);
		mc_record_order( cluster_temp, state.n_particles, "cluster"); 
		mc_record_order( nei_temp, state.n_particles,"neighbours");
		mc_record_order( bond_temp, state.n_particles, "bond_order"); 
		UMB_scale(&umbrella, 0.0, 0);
		UMB_print(&umbrella);
		free(cluster_temp);
	}

	free(en_temp);
	free(bond_temp);
	free(nuclei_int);
	free(nei_temp);
	free(ang_temp);

	//just in case rounding errors//
	if (count_records > (max_steps/average_rate) - 1) break;	
}

/*-------------------------save the data----------------------------*/

fp = fopen("data.txt","w");
fprintf(fp,"Records %d",count_records);
fclose(fp);


mc_save(&state,argv[1],delta_r,0.0,0.0);
RA_print_average(&energy_average);
RA_print_average(&num_average);
RA_print_average(&bond_average);
HG_print_histogram(&energy_histogram);
HG_print_histogram(&bond_histogram);
HG_print_histogram(&num_histogram);
HG_print_histogram(&angle_histogram);
EA_print(&en_function);
H2D_print_histogram(&num_eng,get_area(paramsG));
UMB_scale(&umbrella, 0.0, 0);
UMB_print(&umbrella);

CLL_end(&cell);
MCS_end(&state);
RA_destroy(&energy_average);
RA_destroy(&num_average);
RA_destroy(&bond_average);
HG_destroy(&energy_histogram);
HG_destroy(&bond_histogram);
HG_destroy(&num_histogram);
HG_destroy(&angle_histogram);
EA_destroy(&en_function);
H2D_destroy(&num_eng);
UMB_destroy(&umbrella);
READ_destroy(&input);

return 0;
}
