/* This is a mpdule for calculating the bond order parameter for partices on a torus.
 *
 * It assumes that each particle has six nearest neighbours (so it is a measure of defects as well
 * as order.
 *
 * The prefix is BO6E_
 * */

#include"mc_state_h.h"
#include"cell_list_h.h"
#include"std_h.h"
#include"mc_math_h.h"
#include"bond_order6_eggbox_h.h"

/*FUNCTION: bons order calculate: calculates the bond order parameter of each particle
 *
 *Return: pointer to a list of bond_orders
 *input: state: the current state of the system
 *input radius: the radius of the input sphere*/
double * BO6E_eggbox_calculate (Mc_state *state, Mc_cell *cell, double *paramsG, double cutoff) {

/*dummies*/
	int i,j,count,first,nn,n;
	double xo, yo, zo, xi, yi, zi, dist, nox, noy, noz, nix, niy, niz,ni,no,u,v;
	double xd, yd, zd, xref, yref, zref, sref, scale, angle ,dot;
	double RE, IM, order_re =0, order_im = 0;
	int current_cell;

	double *x_list = state->x;
	double *y_list = state->y;
	double *z_list = state->z;
	long *ll = cell->ll;
	long **neighbours = cell->neighbours;
	long *head_of = cell->head_of;
	long *cell_of = cell->of_particle;
	long n_particles = state->n_particles;

	/*return list*/
	double *orders = malloc (sizeof(double)*state->n_particles);

	/*loop over all particles and get the bond order*/
	for (i = 0; i < n_particles; i++) {
		/*set up*/
		RE = 0;
		IM = 0;
		nn = 0;
		first = 1;
		
		xo = x_list[i];
		yo = y_list[i];
		zo = z_list[i];

		nox = -(paramsG[2]*2*M_PI/(paramsG[1]))*
		       cos((2*M_PI*xo/paramsG[0])+M_PI)*sin((2*M_PI*yo/paramsG[1])+M_PI);
		noy = -(paramsG[2]*2*M_PI/(paramsG[1]))*
		        sin((2*M_PI*xo/paramsG[0])+M_PI)*cos((2*M_PI*yo/paramsG[1])+M_PI);
		noz = 1.0;

		no = sqrt(nox*nox+noy*noy+noz*noz);
		nox = nox/no;
		noy = noy/no;
		noz = noz/no;

		/*get neighbours*/
		current_cell = cell_of[i];
		n = head_of[current_cell];
		while (n != i && n!= -1) {
			/*do work*/
			xi = x_list[n];
			yi = y_list[n];
			zi = z_list[n];

			dist = sqrt( (xo-xi)*(xo-xi) + (yo-yi)*(yo-yi) + (zo-zi)*(zo-zi)  );
			if (dist < cutoff) {

				/*do projection*/
				nix = -(paramsG[2]*2*M_PI/(paramsG[1]))*
				        cos((2*M_PI*xo/paramsG[0])+M_PI)*
					sin((2*M_PI*yo/paramsG[1])+M_PI);
				niy = -(paramsG[2]*2*M_PI/(paramsG[1]))*
				        sin((2*M_PI*xo/paramsG[0])+M_PI)*
					cos((2*M_PI*yo/paramsG[1])+M_PI);	
				niz = 1.0;
				ni = sqrt(nix*nix+niy*niy+niz*niz);
				nix = nix/ni;
				niy = niy/ni;
				niz = niz/ni;

				scale=(xo-xi)*nox+(yo-yi)*noy+(zo-zi)*(noy)/(nox*nix+noy*niy+noz*niz);

				xd = xo-xi-scale*nix;
				yd = yo-yi-scale*niy;
				zd = zo-zi-scale*niz;
				nn++;

				dist = sqrt((xd*xd+yd*yd+zd*zd));

				if (first) {
					xref = xd;
					yref = yd;
					zref = zd;
					sref = sqrt(xref*xref+yref*yref+zref*zref);
					first = 0;
				}
				dot = xd*xref+yd*yref+zd*zref;
				if (fabs(dot/(dist*sref)) > 1.0001) die ("math error");
				if (dot/(dist*sref) < -1.0) {
					angle = M_PI;
				} else if (dot/(dist*sref) > 1.0) {
					angle = 0;
				} else {
					angle = acos(dot/(dist*sref));
				}
				RE += cos(6*angle);
				IM += sin(6*angle); 
			}

			n = ll[n];
		}		
		/*skip one*/
		if (n != -1) {
			n = ll[n];
		}
		while (n != -1) {
			/*do work*/
			xi = x_list[n];
			yi = y_list[n];
			zi = z_list[n];

			dist = sqrt( (xo-xi)*(xo-xi) + (yo-yi)*(yo-yi) + (zo-zi)*(zo-zi)  );
			if (dist < cutoff) {
				/*do projection*/
				nix = -(paramsG[2]*2*M_PI/(paramsG[1]))*
				        cos((2*M_PI*xo/paramsG[0])+M_PI)*
					sin((2*M_PI*yo/paramsG[1])+M_PI);
				niy = -(paramsG[2]*2*M_PI/(paramsG[1]))*
				        sin((2*M_PI*xo/paramsG[0])+M_PI)*
					cos((2*M_PI*yo/paramsG[1])+M_PI);	
				niz = 1.0;
				ni = sqrt(nix*nix+niy*niy+niz*niz);
				nix = nix/ni;
				niy = niy/ni;
				niz = niz/ni;

				scale=(xo-xi)*nox+(yo-yi)*noy+(zo-zi)*(noy)/(nox*nix+noy*niy+noz*niz);

				xd = xo-xi-scale*nix;
				yd = yo-yi-scale*niy;
				zd = zo-zi-scale*niz;
				nn++;

				dist = sqrt((xd*xd+yd*yd+zd*zd));

				if (first) {
					xref = xd;
					yref = yd;
					zref = zd;
					sref = sqrt(xref*xref+yref*yref+zref*zref);
					first = 0;
				}
				dot = xd*xref+yd*yref+zd*zref;
				if (fabs(dot/(dist*sref)) > 1.0001) die ("math error");
				if (dot/(dist*sref) < -1.0) {
					angle = M_PI;
				} else if (dot/(dist*sref) > 1.0) {
					angle = 0;
				} else {
					angle = acos(dot/(dist*sref));
				}
				RE += cos(6*angle);
				IM += sin(6*angle); 
			}
			n = ll[n];
		}
		/*do other cells*/
		for (j = 0; j < 26; j++) {
			current_cell = neighbours[cell_of[i]][j];
			if (current_cell == -1) break;
			n = head_of[current_cell];
			while (n != -1) {
				/*do work*/
				xi = x_list[n];
				yi = y_list[n];
				zi = z_list[n];

				dist = sqrt( (xo-xi)*(xo-xi) + (yo-yi)*(yo-yi) + (zo-zi)*(zo-zi)  );
				if (dist < cutoff) {
					/*do projection*/
					/*do projection*/
					nix = -(paramsG[2]*2*M_PI/(paramsG[1]))*
					        cos((2*M_PI*xo/paramsG[0])+M_PI)*
						sin((2*M_PI*yo/paramsG[1])+M_PI);
					niy = -(paramsG[2]*2*M_PI/(paramsG[1]))*
					        sin((2*M_PI*xo/paramsG[0])+M_PI)*
						cos((2*M_PI*yo/paramsG[1])+M_PI);	
					niz = 1.0;
					ni = sqrt(nix*nix+niy*niy+niz*niz);
					nix = nix/ni;
					niy = niy/ni;
					niz = niz/ni;
		
					scale=(xo-xi)*nox+(yo-yi)*noy+(zo-zi)*(noy)/
						(nox*nix+noy*niy+noz*niz);

					xd = xo-xi-scale*nix;
					yd = yo-yi-scale*niy;
					zd = zo-zi-scale*niz;
					nn++;

					dist = sqrt((xd*xd+yd*yd+zd*zd));

					if (first) {
						xref = xd;
						yref = yd;
						zref = zd;
						sref = sqrt(xref*xref+yref*yref+zref*zref);
						first = 0;
					}
					dot = xd*xref+yd*yref+zd*zref;
					if (fabs(dot/(dist*sref)) > 1.0001) die ("math error");
					if (dot/(dist*sref) < -1.0) {
						angle = M_PI;
					} else if (dot/(dist*sref) > 1.0) {
						angle = 0;
					} else {
						angle = acos(dot/(dist*sref));
					}
					RE += cos(6*angle);
					IM += sin(6*angle); 
			}
				n = ll[n];
			}
		}

		/*collect*/
		if (nn >= 2) {
			RE = RE/6;
			IM = IM/6;
			orders[i] = sqrt(RE*RE+IM*IM);
		} else {
			orders[i] = 0;
		}

	}
	return orders;	
}

