/*This library contains a set of fuctions for dealing with cell lists, in molecular
 *simulations. The cell list must be a cube, and is centred at the origin of the (catesian) frame
 * that the simulation is in.
 *
 * the module prefic is CLL_
 */

#include"mc_state_h.h"
#include"std_h.h"
#include"mc_math_h.h"
#include"cell_list_h.h"

/*
 * A function for initialiseing the cell struct. It fills in the neighbours list,
 * the nunmber of cells on a side and the size of the cell.
 *
 * parameter Mc_cell cell: The struct containing all the cell list information
 * double boxS: the size of the simulation cube.
 */

void CLL_init(Mc_cell *cell, double boxS,double cell_min) {

	/*get cell sizes and number*/

	cell->number = (int) boxS/cell_min;
	if (cell->number % 2 == 1) {
		cell->number --;
	}

	if (cell->number == 0) {
		cell->number = 2;
	}

	cell->size = boxS/cell->number;

	/*allocate head of*/
	cell->head_of = malloc(sizeof(long)*pow(cell->number,3));
	int i,j;
	for(i = 0; i < pow(cell->number,3); i++) {
		cell->head_of[i] = -1;
	}

	/*get neighbours*/
	cell->neighbours = malloc(sizeof(long *)*pow(cell->number,3));
	for (i = 0; i < pow(cell->number,3); i++) {
		cell->neighbours[i] = malloc(sizeof(long)*26);
		for (j = 0; j < 26; j++) {
			cell->neighbours[i][j] = -1;
		}
	}

	/*fill in cell neighbours*/

	/*loop over x,y,z combos*/
	int k,x,y,z,target;

	for (i = 0; i < cell->number;i++) {
		for (j = 0; j < cell->number; j++) {
			for (k = 0; k < cell->number; k++) {
				
				target = i + j*cell->number + k*(cell->number)*(cell->number);

				/*right*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][0] = target - i;
				} else {
					cell->neighbours[target][0] = target +1;
				}
				
				/*left*/
				if (i ==  0  ) {
					cell->neighbours[target][1] = target + cell->number -1;
				} else {
					cell->neighbours[target][1] = target - 1;
				}
						
				/*in*/
				if (j == cell->number -1 ) {
					cell->neighbours[target][2] = target - j*cell->number;
				} else {
					cell->neighbours[target][2] = target + 1*cell->number;
				}
				
				/*out*/
				if (j ==  0  ) {
					cell->neighbours[target][3] = target + cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][3] = target - cell->number;
				}

				/*up*/
				if (k == cell->number -1 ) {
					cell->neighbours[target][4] = target - k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][4] = target + 1*cell->number*
											cell->number;
				}
				
				/*down*/
				if (k ==  0  ) {
					cell->neighbours[target][5] = target + cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][5] = target - cell->number*
											cell->number;
				}
		
				/*right+in*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][6] = target - i;
				} else {
					cell->neighbours[target][6] = target +1;
				}
				if (j == cell->number -1 ) {
					cell->neighbours[target][6] = cell->neighbours[target][6] -
										j*cell->number;
				} else {
					cell->neighbours[target][6] = cell->neighbours[target][6] +
										 1*cell->number;
				}

				/*left+in*/		
				if (i ==  0  ) {
					cell->neighbours[target][7] = target + cell->number -1;
				} else {
					cell->neighbours[target][7] = target - 1;
				}
				if (j == cell->number -1 ) {
					cell->neighbours[target][7] = cell->neighbours[target][7] -
										j*cell->number;
				} else {
					cell->neighbours[target][7] = cell->neighbours[target][7] +
										 1*cell->number;
				}

				/*right+out*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][8] = target - i;
				} else {
					cell->neighbours[target][8] = target +1;
				}
				if (j ==  0  ) {
					cell->neighbours[target][8] = cell->neighbours[target][8] +
										 cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][8] = cell->neighbours[target][8] -
										 cell->number;
				}

				/*left+out*/
				if (i ==  0  ) {
					cell->neighbours[target][9] = target + cell->number -1;
				} else {
					cell->neighbours[target][9] = target - 1;
				}
				if (j ==  0  ) {
					cell->neighbours[target][9] = cell->neighbours[target][9] +
										 cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][9] = cell->neighbours[target][9] -
										 cell->number;
				}

				/*right+up*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][10] = target - i;
				} else {
					cell->neighbours[target][10] = target +1;
				}
				if (k == cell->number -1 ) {
					cell->neighbours[target][10] = cell->neighbours[target][10] -
										 k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][10] = cell->neighbours[target][10] +
										 1*cell->number*
											cell->number;
				}

				/*left+up*/
				if (i ==  0  ) {
					cell->neighbours[target][11] = target + cell->number -1;
				} else {
					cell->neighbours[target][11] = target - 1;
				}
				if (k == cell->number -1 ) {
					cell->neighbours[target][11] = cell->neighbours[target][11] -
										 k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][11] = cell->neighbours[target][11] +
										 1*cell->number*
											cell->number;
				}

				/*right+down*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][12] = target - i;
				} else {
					cell->neighbours[target][12] = target +1;
				}
				if (k ==  0  ) {
					cell->neighbours[target][12] = cell->neighbours[target][12] +
									 cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][12] = cell->neighbours[target][12] -
										 cell->number*
											cell->number;
				}

				/*left+down*/
				if (i ==  0  ) {
					cell->neighbours[target][13] = target + cell->number -1;
				} else {
					cell->neighbours[target][13] = target - 1;
				}
				if (k ==  0  ) {
					cell->neighbours[target][13] = cell->neighbours[target][13] +
									 cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][13] = cell->neighbours[target][13] -
										 cell->number*
											cell->number;
				}

				/*in+up*/
				if (j == cell->number -1 ) {
					cell->neighbours[target][14] = target - j*cell->number;
				} else {
					cell->neighbours[target][14] = target + 1*cell->number;
				}
				if (k == cell->number -1 ) {
					cell->neighbours[target][14] = cell->neighbours[target][14] -
										 k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][14] = cell->neighbours[target][14] +
										 1*cell->number*
											cell->number;
				}

				/*out+up*/
				if (j ==  0  ) {
					cell->neighbours[target][15] = target + cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][15] = target - cell->number;
				}
				if (k == cell->number -1 ) {
					cell->neighbours[target][15] = cell->neighbours[target][15] -
										 k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][15] = cell->neighbours[target][15] +
										 1*cell->number*
											cell->number;
				}

				/*in+down*/
				if (j == cell->number -1 ) {
					cell->neighbours[target][16] = target - j*cell->number;
				} else {
					cell->neighbours[target][16] = target + 1*cell->number;
				}
				if (k ==  0  ) {
					cell->neighbours[target][16] = cell->neighbours[target][16] +
									 cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][16] = cell->neighbours[target][16] -
										 cell->number*
											cell->number;
				}

				/*out+down*/
				if (j ==  0  ) {
					cell->neighbours[target][17] = target + cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][17] = target - cell->number;
				}
				if (k ==  0  ) {
					cell->neighbours[target][17] = cell->neighbours[target][17] +
									 cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][17] = cell->neighbours[target][17] -
										 cell->number*
											cell->number;
				}

				/*right+in+up*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][18] = target - i;
				} else {
					cell->neighbours[target][18] = target +1;
				}
				if (j == cell->number -1 ) {
					cell->neighbours[target][18] = cell->neighbours[target][18] -
										j*cell->number;
				} else {
					cell->neighbours[target][18] = cell->neighbours[target][18] +
										 1*cell->number;
				}
				if (k == cell->number -1 ) {
					cell->neighbours[target][18] = cell->neighbours[target][18] -
										 k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][18] = cell->neighbours[target][18] +
										 1*cell->number*
											cell->number;
				}

				/*right+in+down*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][19] = target - i;
				} else {
					cell->neighbours[target][19] = target +1;
				}
				if (j == cell->number -1 ) {
					cell->neighbours[target][19] = cell->neighbours[target][19] -
										j*cell->number;
				} else {
					cell->neighbours[target][19] = cell->neighbours[target][19] +
										 1*cell->number;
				}
				if (k ==  0  ) {
					cell->neighbours[target][19] = cell->neighbours[target][19] +
									 cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][19] = cell->neighbours[target][19] -
										 cell->number*
											cell->number;
				}

				/*right+out+up*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][20] = target - i;
				} else {
					cell->neighbours[target][20] = target +1;
				}
				if (j ==  0  ) {
					cell->neighbours[target][20] = cell->neighbours[target][20] +
										 cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][20] = cell->neighbours[target][20] -
										 cell->number;
				}
				if (k == cell->number -1 ) {
					cell->neighbours[target][20] = cell->neighbours[target][20] -
										 k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][20] = cell->neighbours[target][20] +
										 1*cell->number*
											cell->number;
				}

				/*right+out+down*/
				if (i == cell->number -1 ) {
					cell->neighbours[target][21] = target - i;
				} else {
					cell->neighbours[target][21] = target +1;
				}
				if (j ==  0  ) {
					cell->neighbours[target][21] = cell->neighbours[target][21] +
										 cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][21] = cell->neighbours[target][21] -
										 cell->number;
				}
				if (k ==  0  ) {
					cell->neighbours[target][21] = cell->neighbours[target][21] +
									 cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][21] = cell->neighbours[target][21] -
										 cell->number*
											cell->number;
				}

				/*left+in+up*/
				if (i ==  0  ) {
					cell->neighbours[target][22] = target + cell->number -1;
				} else {
					cell->neighbours[target][22] = target - 1;
				}
				if (j == cell->number -1 ) {
					cell->neighbours[target][22] = cell->neighbours[target][22] -
										j*cell->number;
				} else {
					cell->neighbours[target][22] = cell->neighbours[target][22] +
										 1*cell->number;
				}
				if (k == cell->number -1 ) {
					cell->neighbours[target][22] = cell->neighbours[target][22] -
										 k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][22] = cell->neighbours[target][22] +
										 1*cell->number*
											cell->number;
				}

				/*left+in+down*/
				if (i ==  0  ) {
					cell->neighbours[target][23] = target + cell->number -1;
				} else {
					cell->neighbours[target][23] = target - 1;
				}
				if (j == cell->number -1 ) {
					cell->neighbours[target][23] = cell->neighbours[target][23] -
										j*cell->number;
				} else {
					cell->neighbours[target][23] = cell->neighbours[target][23] +
										 1*cell->number;
				}
				if (k ==  0  ) {
					cell->neighbours[target][23] = cell->neighbours[target][23] +
									 cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][23] = cell->neighbours[target][23] -
										 cell->number*
											cell->number;
				}

				/*left+out+up*/
				if (i ==  0  ) {
					cell->neighbours[target][24] = target + cell->number -1;
				} else {
					cell->neighbours[target][24] = target - 1;
				}
				if (j ==  0  ) {
					cell->neighbours[target][24] = cell->neighbours[target][24] +
										 cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][24] = cell->neighbours[target][24] -
										 cell->number;
				}
				if (k == cell->number -1 ) {
					cell->neighbours[target][24] = cell->neighbours[target][24] -
										 k*cell->number*
											cell->number;
				} else {
					cell->neighbours[target][24] = cell->neighbours[target][24] +
										 1*cell->number*
											cell->number;
				}

				/*left+out+down*/
				if (i ==  0  ) {
					cell->neighbours[target][25] = target + cell->number -1;
				} else {
					cell->neighbours[target][25] = target - 1;
				}
				if (j ==  0  ) {
					cell->neighbours[target][25] = cell->neighbours[target][25] +
										 cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][25] = cell->neighbours[target][25] -
										 cell->number;
				}
				if (k ==  0  ) {
					cell->neighbours[target][25] = cell->neighbours[target][25] +
									 cell->number*
										cell->number*
										(cell->number -1);
				} else {
					cell->neighbours[target][25] = cell->neighbours[target][25] -
										 cell->number*
											cell->number;
				}
				
			}
		}
	}
	
	double temp_list[26];
	int dup;
	/*remove duplicates*/
	for (i = 0; i < pow(cell->number,3); i++) {
		/*check each element and remove duplicates*/
		for (j = 0; j < 26; j++) {
			temp_list[j] = -2;			
		}

		for (j = 0; j < 26; j++) {
			if (cell->neighbours[i][j] == -1) break;
			/*is it in temp_list*/
			dup = 0;
			for (k = 0; k < 26; k++) {
				if (cell->neighbours[i][j] == temp_list[k]) {
					dup = 1;
				}
			} 

			/*if no, add to temp_list. Remove otherwise*/
			if (dup == 0 && cell->neighbours[i][j] != i) {
				for (k = 0; k < 26; k++) {
					if (temp_list[k] == -2 || 
							temp_list[k] == cell->neighbours[i][j]) {

						temp_list[k] = cell->neighbours[i][j];
						break;
					}
				}
			} else {
				for (k = j; k < 25; k++) {
					cell->neighbours[i][k] = cell->neighbours[i][k+1];
				}
				cell->neighbours[i][25] = -1;
				j--;
			}
		}
	
	}
}

/*perform a deepcopy of cellI into cellT
 *
 *param cellT the target cell to be copies into. Current contents are overwritten
 *input cellI the cell list to be used as the source of the copy (not editied)
 *n_new, the new number of particles to go into cellT
 *n old the old number of particlesint cellT
 */


void CLL_cell_deepCopy (Mc_cell *cellT, Mc_cell *cellI, int n_new, int n_old) {
	
	int i;
	long *temptr;

	if (n_new == n_old) {
		for (i = 0 ; i < n_new; i++) {
			cellT->of_particle[i] = cellI->of_particle[i];
			cellT->ll[i] = cellI->ll[i];
		}
		for (i = 0; i < pow(cellT->number,3); i++) {
			cellT->head_of[i] = cellI->head_of[i];
		}
	} else {
		temptr = realloc(cellT->of_particle, sizeof(long)*n_new);
		if (temptr != NULL || n_new == 0) {
			cellT->of_particle = temptr;
		} else die("realloc failed in cell deep copy");
		temptr = realloc (cellT->ll, sizeof(long)*n_new);
		if (temptr != NULL || n_new == 0) {
			cellT->ll = temptr;
		} else die("realloc failed in cell deep copy");
		for (i = 0 ; i < n_new; i++) {
			cellT->of_particle[i] = cellI->of_particle[i];
			cellT->ll[i] = cellI->ll[i];
		}
		for (i = 0; i < pow(cellT->number,3); i++) {
			cellT->head_of[i] = cellI->head_of[i];
		}

	}
}

/*clones cellI into the (uninitialised struct) cellT
 *param cellT the target cell to be copies into. Current contents are overwritten
 *input cellI the cell list to be used as the source of the copy (not editied)
 *n, the new number of particles in cellI
 */
void CLL_cell_clone (Mc_cell *cellT, Mc_cell *cellI, int n) {
	int i,j;
	
	cellT->of_particle = malloc(sizeof(long)*n);
	cellT->head_of = malloc(sizeof(long)*pow(cellI->number,3));
	cellT->ll = malloc(sizeof(long)*n);
	
	/*get neighbours*/
	cellT->neighbours = malloc(sizeof(long *)*pow(cellI->number,3));
	for (i = 0; i < pow(cellI->number,3); i++) {
		cellT->neighbours[i] = malloc(sizeof(long)*26);
	}

	/*copy*/
	cellT->size = cellI->size;
	cellT->number = cellI->number;
	
	for (i = 0; i < n ; i++) {
		cellT->of_particle[i] = cellI->of_particle[i];
		cellT->ll[i] = cellI->ll[i];
	}

	for (i = 0; i < pow(cellI->number,3); i++) {
		cellT->head_of[i] = cellI->head_of[i];
		for (j = 0; j < 26; j++) {
			cellT->neighbours[i][j] = cellI->neighbours[i][j];
		}
	} 
	
}

/*
 * A fuction for putting the initial group of particles into the cell list correctly
 *
 * param MC_cell cell
 * input Mc_state state: the stare struct of the particles in the system
 */

void CLL_init_particles(Mc_cell *cell, Mc_state *state) {

	/*allocate arrays*/
	cell->ll = malloc(sizeof(long)*state->n_particles);
	int i,n;
	for (i = 0; i < state->n_particles; i++) {
		cell->ll[i] = -1;
	}

	cell->of_particle = malloc(sizeof(long)*state->n_particles);

	/*place particles*/
	long xcell, ycell, zcell, Tcell;
	int half_no = cell->number/2;

	for (i = 0; i < state->n_particles; i++) {
		xcell = floor(state->x[i]/cell->size) + half_no;
		ycell = floor(state->y[i]/cell->size) + half_no;
		zcell = floor(state->z[i]/cell->size) + half_no;
		Tcell = xcell + ycell*cell->number + zcell*(cell->number)*(cell->number);
		cell->of_particle[i] = Tcell;

		if (cell->head_of[Tcell] == -1) {
			cell->head_of[Tcell] = i;
		} else {
			n = cell->head_of[Tcell];
			while (cell->ll[n] != -1) {
				n = cell->ll[n];
			}
			cell->ll[n] = i;
		}
	}
}

/*
 * A function for updating the cell of a single particle
 *
 * param Mc_cell: the cell ist to be updated
 * input Mc_state: the current state of the system
 * input int: the index of the particle to be updated
 */

void CLL_update_particle(Mc_cell *cell, Mc_state *state, int index) {

	long Ocell;
	int n;
	int half_no = cell->number/2;
	int xcell = floor(state->x[index]/cell->size) + half_no;
	int ycell = floor(state->y[index]/cell->size) + half_no;
	int zcell = floor(state->z[index]/cell->size) + half_no;
	long Tcell = xcell + ycell*cell->number + zcell*(cell->number)*(cell->number);
	if (cell->of_particle[index] == Tcell) {
		return;
	} else {
		Ocell = cell->of_particle[index];
		/*remove cell from old lists*/
		if (cell->head_of[Ocell] == index) {
			cell->head_of[Ocell] = cell->ll[index];
		} else {
			n = cell->head_of[Ocell];
			while(cell->ll[n] != index) {
				n = cell->ll[n];
			}
			cell->ll[n] = cell->ll[index];
		}
		/*add particle to new list*/
		if (cell->head_of[Tcell] == -1) {
			cell->head_of[Tcell] = index;
			cell->ll[index] = -1;
		} else {
			n = cell->head_of[Tcell];
			while(cell->ll[n] != -1) {
				n = cell->ll[n];
			}
			cell->ll[n] = index;
			cell->ll[index] = -1;
		}
		cell->of_particle[index] = Tcell;
	}

}

/*a function for adding a particle to the cell list. Assumes it is already in Mc_state
 * ie. call this function after adding the particle
 *
 * A function for updating the cell of a single particle
 *
 * param Mc_cell: the cell ist to be updated
 * input Mc_state: the current state of the system
 * input int: the index of the particle to be updated
 *
 **/

void CLL_add_particle(Mc_cell *cell, Mc_state *state, int index) {

	/*reallocate arrays*/
	long *temptr, *temptr2;
	temptr = realloc(cell->of_particle,sizeof(long)*state->n_particles);
	if (temptr != NULL) {
		cell->of_particle = temptr;
	} else {
		die("realloc failed in CLL_add_particle");
	}
	temptr = realloc(cell->ll,sizeof(long)*state->n_particles);
	if (temptr != NULL) {
		cell->ll = temptr;
	} else {
		die("realloc failed in CLL_add_particle");
	}	

	long Tcell;
	int n;
	double x,y,z;
	/*get x,y,z and totalCell*/
	int half_no = cell->number/2;
	int xcell = floor(state->x[index]/cell->size) + half_no;
	int ycell = floor(state->y[index]/cell->size) + half_no;
	int zcell = floor(state->z[index]/cell->size) + half_no;

	Tcell = xcell + ycell*cell->number + zcell*(cell->number)*(cell->number);

	/*add particle to new list*/
	if (cell->head_of[Tcell] == -1) {
		cell->head_of[Tcell] = index;
		cell->ll[index] = -1;
	} else {
		n = cell->head_of[Tcell];
		while(cell->ll[n] != -1) {
			n = cell->ll[n];
		}
		cell->ll[n] = index;
		cell->ll[index] = -1;
	}
	cell->of_particle[index] = Tcell;

}


/*a function for removing a particle from the cell list. Assumes it is still in MC_state
 * ie. call before removing the particle from the state struct!!!
 * A function for updating the cell of a single particle
 *
 * param Mc_cell: the cell ist to be updated
 * input Mc_state: the current state of the system
 * input int: the index of the particle to be updated
 *
 **/

void CLL_remove_particle(Mc_cell *cell, Mc_state *state, int index) {

	long Ocell = cell->of_particle[index];
	int n;

	/*remove cell from old connections*/
	/*check if particle was head of Ocell*/
	if (cell->head_of[Ocell] == index) {
		cell->head_of[Ocell] = cell->ll[index];
	} else {
		n = cell->head_of[Ocell];
		while (cell->ll[n] != index) {
			n = cell->ll[n];
		}
		cell->ll[n] = cell->ll[index];
	}

	/*remove particle from lists*/
	int i;
	for (i = index; i < state->n_particles-1; i++) {
		cell->of_particle[i] = cell->of_particle[i+1];
		cell->ll[i] = cell->ll[i+1];
	}
	for (i = 0; i < state->n_particles-1; i++) {
		if (cell->ll[i] > index) {
			cell->ll[i] -= 1;
		}
	}

	for (i = 0; i < pow((cell->number),3);i++) {
		if (cell->head_of[i] > index) {
			cell->head_of[i] -= 1;
		}
	}

	long *temptr, *temptr2;
	temptr = realloc(cell->of_particle,sizeof(long)*(state->n_particles-1));
	if (temptr != NULL) {
		cell->of_particle = temptr;
	} else if (state->n_particles == 1 ) {
		cell->of_particle = temptr;
	} else {
		die("realloc failed in CLL_remove_particle");
	}
	temptr2 = realloc(cell->ll,sizeof(long)*(state->n_particles-1));
	if (temptr != NULL) {
		cell->ll = temptr2;
	} else if (state->n_particles == 1 ) {
		cell->ll = temptr2;
	} else {
		die("realloc failed in CLL_remove_particle");
	}

}



/* a Fuction to get the index of the cell that an imaginary particle at u,v would be in. Does not 
 * edit the cell list.
 *
 * Output: the cell index
 * input: x,y,z the generalised coords of u and v
 */

long CLL_get_index(double x, double y, double z, Mc_cell *cell) {

	int half_no = cell->number/2;
	int xcell = floor(x/cell->size) + half_no;
	int ycell = floor(y/cell->size) + half_no;
	int zcell = floor(z/cell->size) + half_no;

	return xcell + ycell*cell->number + zcell*(cell->number)*(cell->number);

}

/*destroys a cell struct*/
void CLL_end(Mc_cell *cell) {

	int i;
	for (i = 0; i < pow(cell->number,3); i++) {
		free(cell->neighbours[i]);
	}
	free(cell->neighbours);
	free(cell->ll);
	free(cell->head_of);
	free(cell->of_particle);

}





