#ifndef GEOMETRY_H
#define GEOMETRY_H

/*A module for generating trial moves on different surfaces for monte carlo simulation.
 * The distribution of moves is garunteed to be uniform
 * the module prefix is GEO_
 */

/*Function GEO_mv_sphere: generates well sampled purterbations of a particle confined to a sphere
 * params x,y,z: the coordinates of the particle (to be updated)
 * input: *radius pointer to the radius of the sphere 
 * input: delta_r, the amount by which the position can change
 */

void GEO_mv_sphere (double *x, double *y, double *z, double *radius, double delta_r) ;

/*gets a point at random, uniformly distributed on the sphere
 *
 * parms (x,y,x) the coords of the particle
 * input: radius: the radius of the particle
 */

void GEO_random_sphere (double *x, double *y, double *z, double *radius);

/*get the area of a sphere of a given radius*/
double GEO_area_sphere(double *radius);

/*FUNCTION GEO_mv_plane: generates well sampled moves of a particle confined to the x-y plane
 *
 * params x,y,z: the coordinates of the particle (to be updated)
 * params: the x,y lengths of the box (box is centred on the origin
 * input: delta: the amount theat the particle can move in a give ndirection
 */

void GEO_mv_plane(double *x, double *y, double *z, double *params, double delta);

/*FUNCTION GEO_radom plane: randomly places a particle in a box
 *
 * Params x,y,z: the output location of the particel
 * params: the x,y lengths of the box (box is centred on the origin
 */

void GEO_random_plane(double *x, double *y, double *z, double *params);

/*get the size of a plane with a given edge length*/
double GEO_area_plane(double *params);

/*FUNCTION GEO_mv_cone: generates well sampled moves of a particle confined to a cone
 *
 * params x,y,z: the coordinates of the particle (to be updated)
 * params: r,h to describe the cone
 * input: delta: the amount theat the particle can move in either angle
 */

void GEO_mv_cone(double *x, double *y, double *z, double *params, double delta);

/*FUNCTION GEO_radom cone: randomly places a particle on a cone
 *
 * Params x,y,z: the output location of the particel
 * params: a,c to describe the cone
 */
void GEO_random_cone(double *x, double *y, double *z, double *params);

/*get the size of a cone with any parameters*/
double GEO_area_cone(double *params);

/*FUNCTION GEO_mv_torus: generates well sampled moves of a particle confined to a torus
 *
 * params x,y,z: the coordinates of the particle (to be updated)
 * params: a,c to describe the torus
 * input: delta: the amount theat the particle can move in either angle
 */
void GEO_mv_torus(double *x, double *y, double *z, double *params, double delta);

/*FUNCTION GEO_radom torus: randomly places a particle on a torus
 *
 * Params x,y,z: the output location of the particel
 * params: a,c to describe the torus
 */
void GEO_random_torus(double *x, double *y, double *z, double *params);

/*get the size of a torus with any parameters*/
double GEO_area_torus(double *params);

/*FUNCTION GEO move eggbox
 *
 * params x,y,z: the coordinates of the particle (to be updated)
 * params L,L,A to describe the eggbox
 * input delta: the amount the particle can move in x and y
 */
void GEO_mv_eggbox(double *x, double *y, double *z, double *params, double delta);

/* FUNCTION GEO random eggbox
 *
 * params x,y,z: the coordinates of the particle (to be updated)
 * params L,L,A to describe the eggbox
 */
void GEO_random_eggbox(double *x, double *y, double *z, double *params);

/*get the size of the eggbox MASSIVE HACK AGGGGHHHHh*/
double GEO_area_eggbox(double *params);

/*Function: GEO_distance get s the real distance between two points in space
 *
 *output: the distance
 *input: the coordinates in cartesian space of particles 1 and 2
 */
double GEO_distance (double x1, double y1, double z1, double x2, double y2, double z2,
			int periodic, double *params);

#endif
