#include"std_h.h"
#include"histograms_2D_continuous_h.h"

/*This is a module for building two dimensional histograms where the first dimension 
 * is just a number (like number of particles) and the second is a scalar variable (such as energy)
 *
 * The module prefix if H2D_
 */

/*
/*
 * FUNCTION: Constructor: initialise a new histogram
 *
 * double var_max1,2: the maximum value of the variable
 * double var_min1,2: the minimum value of the variable
 * double bin_size1,2: the (maximum) size of the bins for the variable
 * name: string the name of the histogram
 */

void H2DC_new_histogram(Mc_hist2DC *histogram, double max1, double min1, double bin_size1,
			double max2, double min2, double bin_size2 ,char *name, int restart) {

	histogram->n_bins1 = (int) (max1-min1)/bin_size1;
	histogram->bin_size1 = (max1-min1)/histogram->n_bins1;
	histogram->max1 = max1;
	histogram->min1 = min1;
	histogram->n_bins2 = (int) (max2-min2)/bin_size2;
	histogram->bin_size2 = (max2-min2)/histogram->n_bins2;
	histogram->max2 = max2;
	histogram->min2 = min2;
	histogram->freq = malloc(sizeof(long)*histogram->n_bins1*histogram->n_bins2);
	strcpy(histogram->name,name);

	int i,j;
	for(i = 0; i < histogram->n_bins1; i++) {
		for(j =0; j < histogram->n_bins2;j++){
			histogram->freq[i*histogram->n_bins2+j] = 0;
		}
	}
	if (restart) {
		char command[50];
		sprintf(command,"%s.dat",histogram->name);
		FILE *fp;
		char *line = NULL;
		size_t len = 0;
		ssize_t read;
		fp = fopen(command,"r");
		int first = 1;
		long val;
		
		while ((read = getline(&line, &len,fp)) != -1) {

			if (first) {
				first = 0;
			} else {
				sscanf(line,"%d %d %ld %*[\n]",&i,&j,&val);
				histogram->freq[i*histogram->n_bins2+j] = val;	
			}
		}
				

		fclose(fp);		
	}

}

/*FUNCTION: destroy: deallocate the histogram*/
void H2DC_destroy(Mc_hist2DC *histogram) {
	free(histogram->freq);
}

/*Function: add value: adds a single value to the histogram
 *
 *param histogram: the histogram to be added to
 *input: double var: the value of the variable
 */
void H2DC_add_value(Mc_hist2DC *histogram, double var1, double var2){

	double place = histogram->min1 + histogram->bin_size1;
	int i,j,i0,j0;
	i0 = histogram->n_bins1;
	j0 = histogram->n_bins2;
	for (i = 0; i < histogram->n_bins1;i++) {
		if(var1 < place) {
			i0 = i;
			break;
		} else {
			place += histogram->bin_size1;
		}
	}
	place = histogram->min2 + histogram->bin_size2;
	for (j = 0; j < histogram->n_bins2;j++) {
		if(var2 < place) {
			j0=j;
			break;
		} else {
			place += histogram->bin_size2;
		}
	}
	histogram->freq[i0*histogram->n_bins2 + j0] ++;
}

/*Function: add array
 *
 *param histogram: the histogram to be added to
 *input: double var: the value of the variable
 */

void H2DC_add_array(Mc_hist2DC *histogram, double *var1, double *var2,int number) {
	int i;
	for (i = 0; i < number;i++) {
		H2DC_add_value(histogram,var1[i],var2[i]);
	}
}


/*FUNCTION: print: prints the histogram to a file called NAME.dat
 *
 *input : the histogram to be printed
 *input volume: the volume of the system (or area in 2D)
 */
void H2DC_print_histogram(Mc_hist2DC *histogram) {
	char command[50];
	FILE *fp;
	int i,j;
	
	sprintf(command,"%s.dat",histogram->name);
	fp = fopen(command,"w");
	for(i = 0; i < histogram->n_bins1; i++) {
		for(j =0; j < histogram->n_bins2;j++){
			fprintf(fp,"%lf %lf %ld\n",i*histogram->bin_size1,
				j*histogram->bin_size2,histogram->freq[i*histogram->n_bins2+j]);
		}
	}
	fclose(fp);
}


