#ifndef HISTOGRAM2DC_H
#define HISTOGRAM2DC_H

/* a module for dealing with 2D histograms where both variables are continuous
 * prefix is H2DC_
 */

typedef struct hg2Dcmcstrc {

	double bin_size1;
	double max1;
	double min1;
	double bin_size2;
	double max2;
	double min2;
	int n_bins1;
	int n_bins2;
	long *freq;
	char name[50];	
} Mc_hist2DC;

/*
 * FUNCTION: Constructor: initialise a new histogram
 *
 * double var_max1,2: the maximum value of the variable
 * double var_min1,2: the minimum value of the variable
 * double bin_size1,2: the (maximum) size of the bins for the variable
 * name: string the name of the histogram
 */

void H2DC_new_histogram(Mc_hist2DC *histogram, double max1, double min1, double bin_size1,
			double max2, double min2, double bin_size2 ,char *name, int restart);

/*FUNCTION: destroy: deallocate the histogram*/
void H2DC_destroy(Mc_hist2DC *histogram);

/*Function: add value: adds a single value to the histogram
 *
 *param histogram: the histogram to be added to
 *input: double var: the value of the variable
 */
void H2DC_add_value(Mc_hist2DC *histogram, double var1, double var2);

/*Function: add array
 *
 *param histogram: the histogram to be added to
 *input: double var: the value of the variable
 */

void H2DC_add_array(Mc_hist2DC *histogram, double *var1, double *var2,int number);

/*FUNCTION: print: prints the histogram to a file called NAME.dat
 *
 *input : the histogram to be printed
 *input volume: the volume of the system (or area in 2D)
 */
void H2DC_print_histogram(Mc_hist2DC *histogram);

#endif
