#ifndef MORSE_TS_H
#define MORSE_TS_H

/* a module for implementing the truncated, shifted and smoothed Morse potential
 *
 * The prefix is mts_
 *
 */

/*FUNCTION: get params: A helper function to set up the params array for this function to be
 * passed. Not required but useful, as it does all the differentiating for you etc.
 *
 * Output: double params[] (array of size 5)
 *		params[0] = epsilon, the inverse temperature in reduced units
 *		params[1] = rho, the value of the range parameter
 *		params[2] = The cutoff distance for the potential. The unit of distance is to
 *			the distance at which the Lennard Jones potential
 *			has it's minimum.
 *		params[3] = the value of the Lennard jones potential at the cutoff
 *		params[4] = the value of the differential of the potential
 *				at the cutoff
 *
 * Input: double epsilon: The value of the inverse temperature
 * Input double rho: thwe value of the range parameter
 * input double cutoff: the cutoff length. 	
 */

void mts_get_params(double params[], double epsilon, double rho, double cutoff);

/*
 FUNCTION: double morse: calculates the pairwise morse potential
 * OUTPUT: ld potential in units of kT
 * INPUT: double distance the separation of the two particles
 */

double mts_potential(double distance, double params[]);


#endif
