/*a minimodule to deal with the nearest neighbours of a system of particles#
 *
 * the prefix is NEI_
 */

#include"std_h.h"
#include"mc_state_h.h"
#include"geometry_h.h"
#include"neighbours_h.h"

/*function: neighbours calculate: gets the number of nearest neighbours of 
 *all the particles in a state
 *
 *Return: pointer to a list of neighbours
 *input: state: the current state of the system
 *input *param: a param (not used)
 *input cutoff: the proximity of a nearest neighbour
 */

int * NEI_int_calculate (Mc_state *state, double *param, double cutoff, int periodic) {

	int *orders = malloc (sizeof(int) * state->n_particles);
	
	int i,j;
	for (i = 0; i < state->n_particles; i++) {
		orders[i] = 0;
	}
	double dist;

	for (i = 0; i < (state->n_particles)-1; i++) {
		for (j = i+1;j<state->n_particles;j++) {
			dist = GEO_distance(state->x[i],state->y[i],state->z[i],
							state->x[j],state->y[j],state->z[j],
							periodic, param);

			if (dist < cutoff) {
				orders[i]++;
				orders[j]++;
			}
		}
	}

	return orders;

}

/*function: double neighbours: a version of neighbours calculate that returns doubles rather than
 * ints
 *
 */
double * NEI_double (Mc_state *state, double *param, double cutoff, int periodic) {

	double *orders = malloc (sizeof(double) * state->n_particles);
	
	int i,j;
	for (i = 0; i < state->n_particles; i++) {
		orders[i] = 0.0;
	}
	double dist;

	for (i = 0; i < (state->n_particles)-1; i++) {
		for (j = i+1;j<state->n_particles;j++) {
			dist = GEO_distance(state->x[i],state->y[i],state->z[i],
							state->x[j],state->y[j],state->z[j],
							periodic, param);

			if (dist < cutoff) {
				orders[i] += 1.0;
				orders[j] += 1.0;
			}
		}
	}

	return orders;

}

/*function: gets the number of particles in the liquid like state
 * return number of liquid particles
  *input: state: the current state of the system
  *input d_cutoff: the proximity of a nearest neighbour
  *input n_cutoff: the required number of neighbours
*/
double NEI_liquid(Mc_state *state, double *param, double d_cutoff, double n_cutoff, int periodic) {
	int *neighbours;
	neighbours = NEI_int_calculate(state,param,d_cutoff,periodic);
	int i;
	int sum = 0;
	for (i = 0; i < state->n_particles;i++) {
		if(neighbours[i] > n_cutoff) sum++;	
	}
	free(neighbours);
	return (double) sum;	
}

