#ifndef READ_H
#define READ_H


/*  a module to handle the inputs and outputs for the monte carlo simulations
 *  It should work with any of the permutation (parelle temperaing etc)
 *
 *  The module prefix is READ_
 */
#include"mc_state_h.h"
#include"cell_list_h.h"
#include"cluster_moves_h.h"
/*a struct to contains all the input parameters, pass it around etc*/

typedef struct reinp {

	/*geometry*/
	char geometry[50];
	double radius;
	double box[2];
	double torusA;
	double torusC;
	double height;
	int periodic;
	double *paramsG;
	double delta_r;
	double deltaC_move;
	double deltaC_spin;
	void (*move)(double *x, double *y, double *z, double *paramsD, double delta_r);
	void (*move_cluster) (Mc_state *state,Mc_cell *cell,Mc_clusters *cluster,
					int head,double delta,double *paramsG);
	void (*spin_cluster) (Mc_state *state,Mc_cell *cell,Mc_clusters *cluster,
					int head,double delta,double *paramsG);
	void (*random) (double *x, double *y, double *z, double *params);
	double (*get_area) (double *params);

	/*potential*/
	char potential_name[50];
	double epsilon;
	double rho;
	double lambda_scale;
	double alpha;
	double gamma;
	double potential_cutoff;
	double *paramsP;
	double (*potential) (double distance, double params[]);

	/*cell list*/
	double cell_box_edge;
	double cell_min_size;

	/*ensemble*/
	double activity;	

	/*measurements*/
	double neighbour_cutoff;
	double * (*BO6_calculate) (Mc_state *state, Mc_cell *cell, double *paramsG, double cutoff);

	/*general*/
	long n_particles_initial;
	long read_file;
	long restart;
	long seed;
	
	long max_steps;
	long equilibriate;
	long frame_rate;
	long record_rate;
	long average_rate;

	/*umbrella*/
	double lambda;
	long n_target;
	long n_max;
	long umbrella_rate;
	double order_cutoff;

} Data;

/*
 * Function READ_options: reads in the file input and populates
 * it with the keywords. It does some validation checks and so on as well.
 */

void READ_options(Data *input, FILE *options);

/*deallocate a read struct*/
void READ_destroy(Data *input);

#endif
