/*A module to handle replica exchange parellel tempering, initially, ony "1D" replica
 exchanges are inplemented. the prefix is RE_*/

#include"std_h.h"
#include"mpi_h.h"
#include"mc_math_h.h"
#include"replica_exchange_h.h"
#include"twister_h.h"
#include"mc_state_h.h"

/*initialises the mpi library, and gets the rank and soze variables*/
void RE_initialise (int *rank, int *size) {

	MPI_Init(NULL,NULL);
	MPI_Comm_rank(MPI_COMM_WORLD, rank);
	MPI_Comm_size(MPI_COMM_WORLD, size);

}

/*ends the parellelisation*/
void RE_end (void) {
	MPI_Finalize();
}

/*Function: MC_save: saves the current state of the system: can be used for restarts*/
void RE_mc_save(Mc_state *state, char run_id[], double delta_r, double deltaC, double deltaS) {
	FILE  *fp;
	int i;
	char command[50];
	sprintf(command,"../run%s/state%s.csv",run_id,run_id);
	fp = fopen(command,"w");
	fprintf(fp,"%ld\n",state->n_particles);
	fprintf(fp,"%lf\n",delta_r);
	fprintf(fp,"%lf\n",deltaC);
	fprintf(fp,"%lf\n",deltaS);
	for (i = 0; i < state->n_particles; i++) {
		fprintf(fp,"%lf,%lf,%lf\n",state->x[i],state->y[i],state->z[i]);
	}
	fclose(fp);
}

/*a function for attempting a replica exchange wit ha process further down the list of replicas
 * this function does not handle accept/reject, which is delt with by the exchange_up function
 *
 * Output: 1 if success else 0
 * Input: var: the system variable need ed for the acceptance check. eg the total energy or
		the particle number etc.
 *param: param: the parameter to be exchangeg eg. temperature
 *param delta[]: move distances to be transfered
 *input: n_delta: number of move distances
 *input rank, current mpi rank
 *iparam i_sim, the index of this simulation 
 */

int RE_exchange_down(double var, double *param, double *delta, int n_delta, int rank, int *i_sim) {
       double *params  = malloc(sizeof(double)*(n_delta + 3));
       double *paramsR = malloc(sizeof(double)*(n_delta + 3));
	params[0] = var;
	params[1] = *param;
	params[2] = *i_sim;
	int i,success;
	for (i = 0; i < n_delta; i++) {
		params[i+3] = delta[i];
	}

	MPI_Status status;

	MPI_Ssend(params,(n_delta+3),MPI_DOUBLE,rank-1,0,MPI_COMM_WORLD);
	MPI_Recv(paramsR,(n_delta+3),MPI_DOUBLE,rank-1,1,MPI_COMM_WORLD,&status);

	*param = paramsR[0];
	*i_sim = (int) paramsR[1];
	for (i = 0; i < n_delta; i++) {
		delta[i] = paramsR[i+3];
	}

	if (paramsR[2] > 0.5 ) {
		success = 1;
	} else {
		success = 0;
	}
	free(params);
	free(paramsR);
	return success;	
}

/*a function for attempting a replica exchange wit ha process further up the list of replicas
 * this function does handle accept/reject.
 * Output: 1 if success else 0
 * Input: var: the system variable need ed for the acceptance check. eg the total energy or
		the particle number etc.
 *param: param: the parameter to be exchangeg eg. temperature
 *param delta[]: move distances to be transfered
 *input: n_delta: number of move distances
 *input rank, current mpi rank
 *iparam i_sim, the index of this simulation
 *function RE_accept:
 */

int RE_exchange_up(double var, double *param, double *delta, int n_delta, int rank, int *i_sim,
			int(*RE_accept)(double var1, double var2, double param1, double param2)){

	int success,i;
	double *params  = malloc(sizeof(double)*(n_delta + 3));
	double *paramsR = malloc(sizeof(double)*(n_delta + 3));

	MPI_Status status;

	MPI_Recv(paramsR,(n_delta+3),MPI_DOUBLE,rank+1,0,MPI_COMM_WORLD,&status);

	if (RE_accept(var,paramsR[0],*param,paramsR[1])) {
		success = 1;
		params[0] = *param;
		params[1] = (double) *i_sim;
		params[2] = (double) success;
		for (i = 0; i < n_delta; i++) {
			params[i+3] = delta[i];
		} 

		*param = paramsR[1];
		*i_sim = paramsR[2];
		for (i = 0; i < n_delta; i++) {
			delta[i] = paramsR[i+3];
	        }
	} else {
		success = 0;
		params[0] = paramsR[1];
		params[1] = paramsR[2];
		params[2] = (double) success;
		for (i = 0; i < n_delta; i++) {
			params[i+3] = paramsR[i+3];
		}
		
	}

	MPI_Ssend(params,n_delta+3,MPI_DOUBLE,rank+1,1,MPI_COMM_WORLD);
	free(params);
	free(paramsR);
	return success;
	
}

/*The acceptence rule for temperature exchanges
 *
 * output 1: accept 0:reject
 * input en1, en2: the energies of the exchanging systems
 * input eps1, eps2: the inverse temperatures of the exchanging systems
 */
int RE_accept_temperature (double en1,double en2,double eps1,double eps2) {
	return (randomF() < exp( (eps1-eps2)*(en1-en2)) );

}

/*The acceptence rule for temperature exchanges
 *
 * output 1: accept 0:reject
 * input num1, num2: the number of particles is sytem one and two
 * input act1 act2: the activities in systems 1 and 2
 */

int RE_accept_activity (double num1, double num2, double act1, double act2) {

	return (randomF() < pow((act1/act2),(num2-num1)) );

}

/*
 *The acceptence rule for n_target exchanges
 *
 * output 1: accept 0:reject
 * input n_lambda1, n_lambda2: the size of the largest cluster times lambda
 * input n_t1, n_t2; the size of the target cluster
 */

int RE_accept_n_target (double n_lambda1, double n_lambda2, double n_t1, double n_t2) {

  return (randomF() < exp ( -1 * (n_lambda1-n_lambda2)*(n_t1-n_t2)  ));

}





