#ifndef UMBRELLA_BASIC_H
#define UMBRELLA_BASIC_H

/*this module simply implemetns the basics of umbrella sampling, without any assumptions about the
 * type of order parameter. Assumes no  replica exchange.
 *
 * the module prefix is _UBB*/

/*Struct: for storing the size of the largest nucleus as a histogram*/

typedef struct umbrstrcb {

	int max_size;
	long *freq;
	double *scaled;
	int isScaled;
	char name[50];
	
} Mc_umbrellaB;

/* FUNCTION: constructor: makes a new histogram big enough to contain all posible nuclear sizes
 * 
 * Input: histogram: the histogram to be initialised
 * Input: Volume: the volume of the system in units of particle volume
 * Input: *name: the name of the histogram
 */
void UBB_new_histogram(Mc_umbrellaB *histogram, double volume, char *name, int restart);

/*Function add value: add a new nucleus to the histogram
 *
 * Input: histogram: the histogram to be initialised
 * Input: value: the value to be added
 **/
void UBB_add_value(Mc_umbrellaB *histogram, int value);

/* Function: scale:  scales the histogram acording to the rules of umbrella sampling
 *
 * Input: Mc_umbrella : the histogram to be scaled
 * Input: lambda: the spring constant of the umbrella potential
 *input : n_target: The target nuclear size
 */

void UBB_scale(Mc_umbrellaB *histogram, double lambda, int n_target);

/* Function: print: prints out both the raw feqecncy and scaled histogram data
 *
 * Input histogramL the histgoram to be printed
 **/

void UBB_print(Mc_umbrellaB *histogram);

/*Destructor*/
void UBB_destroy(Mc_umbrellaB *histogram);


/*FUNCTION UBB_move: This function checks whether the latest batch of umbrella moves is accepted
 * or rejected. This is based on the values of the order parater u_rec and u_old
 *
 * Output: 1 if the move is aacpected, 0 otherwise
 * Param: *mc_state state: the current state of the system
 * Param: *mc_state old_state: the old state of the system
 * Output: u_rec, the final nulcear size for this system
 * param: u_old: the previous size of the largest nucleus in the system
 * input: lambda: the weighting of the umbrealla sampling bias energy
 * input n_target: the target nuclear size */

int UBB_move(Mc_state *state, Mc_state *old_state, Mc_cell *cell, Mc_cell *old_cell, int *u_rec,
		int *u_old, double lambda, int n_target);

/*FUNCTION UBB_move_max: This fuction checks whether the latest batch of umbrella moves
 * is accepted or rejected. All moves with nucleus > n_max get rejected, otherwise accpeted.
 *
 *Output: 1 if the move is accpected, zero otherwise
 *Param: *mc_state state: the current state of the system
 *Param: *mc_state old_state: the old state of the system
 *Output: u_rec, the final nulcear size for this system
 *param: u_old: the previous size of the largest nucleus in the system
 * input n_max: the largest allowed nucleus
 */

int UBB_move_max (Mc_state *state, Mc_state *old_state, Mc_cell *cell, Mc_cell *old_cell,
		  int *u_rec, int *u_old,int n_max);

#endif
